#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <QDrag>
#include <QMimeData>
#include <QDragEnterEvent>
#include <QObjectUserData>

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    ui->toolbox->addTopLevelItem (new ButtonFactory);
    ui->toolbox->addTopLevelItem (new EditFactory);
}

MainWindow::~MainWindow()
{
    delete ui;
}

// #include <QDrag>
// #include <QMimeData>


void MainWindow::on_toolbox_itemPressed (QTreeWidgetItem *item, int column)
{
    QDrag * drag = new QDrag (this);
    QMimeData * mimeData = new QMimeData ();
    QByteArray block ((const char*)item, sizeof (item));
    mimeData->setData ("factory", block);
    drag->setMimeData (mimeData);
    drag->exec (Qt::CopyAction | Qt::MoveAction | Qt::LinkAction);
}

void Area::dragEnterEvent (QDragEnterEvent * event)
{
    if (event->mimeData()->hasFormat("factory"))
       event->setAccepted (true);
}

void Area::dropEvent(QDropEvent * event)
{
    const QMimeData * mimeData = event->mimeData();
    if (mimeData->hasFormat("factory"))
    {
        QByteArray block = mimeData->data ("factory");
        Factory * factory =  (Factory  *) block.data ();
        QWidget * widget = factory->create();

        widget->setParent (this);
        widget->setVisible (true);

        QPoint p = event->pos ();
        QRect r = widget->geometry ();
        widget->setGeometry (p.x(), p.y(), r.width(), r.height());

        widget->setObjectName (QString ("new ") + widget->metaObject()->className());
    }

}

void MainWindow::focusChanged (QWidget * old, QWidget * now)
{
    ui->info->setText (now ? now->objectName() : "");
}
